@@ -106,7 +106,7 @@ nm_keyfile_plugin_kf_set_##stype##_list (GKeyFile *kf, \
 	const char *alias; \
  \
 	alias = nm_keyfile_plugin_get_alias_for_setting_name (group); \
-	g_key_file_set_##stype##_list (kf, alias ? alias : group, key, list, length); \
+	g_key_file_set_##stype##_list (kf, alias ?: group, key, list, length); \
 }
 
 DEFINE_KF_LIST_WRAPPER(integer, gint*, gint);
@@ -170,7 +170,7 @@ nm_keyfile_plugin_kf_set_##stype (GKeyFile *kf, \
 	const char *alias; \
  \
 	alias = nm_keyfile_plugin_get_alias_for_setting_name (group); \
-	g_key_file_set_##stype (kf, alias ? alias : group, key, value); \
+	g_key_file_set_##stype (kf, alias ?: group, key, value); \
 }
 
 DEFINE_KF_WRAPPER(string, gchar*, const gchar*);
@@ -179,7 +179,6 @@ DEFINE_KF_WRAPPER(uint64, guint64, guint64);
 DEFINE_KF_WRAPPER(boolean, gboolean, gboolean);
 DEFINE_KF_WRAPPER(value, gchar*, const gchar*);
 
-
 gchar **
 nm_keyfile_plugin_kf_get_keys (GKeyFile *kf,
                                const char *group,
@@ -288,7 +287,6 @@ _nm_keyfile_a_contains_all_in_b (GKeyFile *kf_a, GKeyFile *kf_b)
 	return TRUE;
 }
 
-
 static gboolean
 _nm_keyfile_equals_ordered (GKeyFile *kf_a, GKeyFile *kf_b)
 {
