@@ -160,4 +160,10 @@
 # define NM_AVAILABLE_IN_1_12
 #endif
 
+#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_12_2
+# define NM_AVAILABLE_IN_1_12_2          G_UNAVAILABLE(1,12,2)
+#else
+# define NM_AVAILABLE_IN_1_12_2
+#endif
+
 #endif  /* NM_VERSION_H */
