@@ -26,6 +26,12 @@
 #ifndef __NM_VPN_DBUS_INTERFACE_H__
 #define __NM_VPN_DBUS_INTERFACE_H__
 
+#include "nm-dbus-interface.h"
+
+#ifndef NM_VERSION_H
+#define NM_DEPRECATED_IN_1_8_FOR(n)
+#endif
+
 /*
  * dbus services details
  */
@@ -54,7 +60,6 @@
 #define NM_DBUS_VPN_BAD_ARGUMENTS             "BadArguments"
 #define NM_DBUS_VPN_INTERACTIVE_NOT_SUPPORTED "InteractiveNotSupported"
 
-
 /*
  * VPN daemon signals
  */
@@ -89,7 +94,6 @@ typedef enum {
 	NM_VPN_SERVICE_STATE_STOPPED
 } NMVpnServiceState;
 
-
 /**
  * NMVpnConnectionState:
  * @NM_VPN_CONNECTION_STATE_UNKNOWN: The state of the VPN connection is
@@ -147,19 +151,20 @@ typedef enum {
  *
  * VPN connection state reasons
  */
+NM_DEPRECATED_IN_1_8_FOR(NMActiveConnectionStateReason)
 typedef enum {
-	NM_VPN_CONNECTION_STATE_REASON_UNKNOWN = 0,
-	NM_VPN_CONNECTION_STATE_REASON_NONE,
-	NM_VPN_CONNECTION_STATE_REASON_USER_DISCONNECTED,
-	NM_VPN_CONNECTION_STATE_REASON_DEVICE_DISCONNECTED,
-	NM_VPN_CONNECTION_STATE_REASON_SERVICE_STOPPED,
-	NM_VPN_CONNECTION_STATE_REASON_IP_CONFIG_INVALID,
-	NM_VPN_CONNECTION_STATE_REASON_CONNECT_TIMEOUT,
-	NM_VPN_CONNECTION_STATE_REASON_SERVICE_START_TIMEOUT,
-	NM_VPN_CONNECTION_STATE_REASON_SERVICE_START_FAILED,
-	NM_VPN_CONNECTION_STATE_REASON_NO_SECRETS,
-	NM_VPN_CONNECTION_STATE_REASON_LOGIN_FAILED,
-	NM_VPN_CONNECTION_STATE_REASON_CONNECTION_REMOVED
+	NM_VPN_CONNECTION_STATE_REASON_UNKNOWN                  = NM_ACTIVE_CONNECTION_STATE_REASON_UNKNOWN,
+	NM_VPN_CONNECTION_STATE_REASON_NONE                     = NM_ACTIVE_CONNECTION_STATE_REASON_NONE,
+	NM_VPN_CONNECTION_STATE_REASON_USER_DISCONNECTED        = NM_ACTIVE_CONNECTION_STATE_REASON_USER_DISCONNECTED,
+	NM_VPN_CONNECTION_STATE_REASON_DEVICE_DISCONNECTED      = NM_ACTIVE_CONNECTION_STATE_REASON_DEVICE_DISCONNECTED,
+	NM_VPN_CONNECTION_STATE_REASON_SERVICE_STOPPED          = NM_ACTIVE_CONNECTION_STATE_REASON_SERVICE_STOPPED,
+	NM_VPN_CONNECTION_STATE_REASON_IP_CONFIG_INVALID        = NM_ACTIVE_CONNECTION_STATE_REASON_IP_CONFIG_INVALID,
+	NM_VPN_CONNECTION_STATE_REASON_CONNECT_TIMEOUT          = NM_ACTIVE_CONNECTION_STATE_REASON_CONNECT_TIMEOUT,
+	NM_VPN_CONNECTION_STATE_REASON_SERVICE_START_TIMEOUT    = NM_ACTIVE_CONNECTION_STATE_REASON_SERVICE_START_TIMEOUT,
+	NM_VPN_CONNECTION_STATE_REASON_SERVICE_START_FAILED     = NM_ACTIVE_CONNECTION_STATE_REASON_SERVICE_START_FAILED,
+	NM_VPN_CONNECTION_STATE_REASON_NO_SECRETS               = NM_ACTIVE_CONNECTION_STATE_REASON_NO_SECRETS,
+	NM_VPN_CONNECTION_STATE_REASON_LOGIN_FAILED             = NM_ACTIVE_CONNECTION_STATE_REASON_LOGIN_FAILED,
+	NM_VPN_CONNECTION_STATE_REASON_CONNECTION_REMOVED       = NM_ACTIVE_CONNECTION_STATE_REASON_CONNECTION_REMOVED,
 } NMVpnConnectionStateReason;
 
 /**
@@ -177,12 +182,18 @@ typedef enum {
 	NM_VPN_PLUGIN_FAILURE_BAD_IP_CONFIG
 } NMVpnPluginFailure;
 
+#ifndef NM_VERSION_H
+#undef NM_DEPRECATED_IN_1_8_FOR
+#endif
 
 /*** Generic config ***/
 
 /* string: VPN interface name (tun0, tap0, etc) */
 #define NM_VPN_PLUGIN_CONFIG_TUNDEV      "tundev"
 
+/* string: Proxy PAC */
+#define NM_VPN_PLUGIN_CONFIG_PROXY_PAC   "pac"
+
 /* string: Login message */
 #define NM_VPN_PLUGIN_CONFIG_BANNER      "banner"
 
@@ -203,7 +214,6 @@ typedef enum {
  */
 #define NM_VPN_PLUGIN_CAN_PERSIST        "can-persist"
 
-
 /*** Ip4Config ***/
 
 /* uint32: IP address of the internal gateway of the subnet the VPN interface is
@@ -260,7 +270,6 @@ typedef enum {
 #define NM_VPN_PLUGIN_IP4_CONFIG_MTU         NM_VPN_PLUGIN_CONFIG_MTU
 #define NM_VPN_PLUGIN_IP4_CONFIG_TUNDEV      NM_VPN_PLUGIN_CONFIG_TUNDEV
 
-
 /*** Ip6Config ***/
 
 /* array of uint8: IP address of the internal gateway of the subnet the VPN interface is
