@@ -43,9 +43,10 @@
  *   and g_test_assert_expected_messages() will not fail.
  *
  * NMTST_SEED_RAND environment variable:
- *   Tests that use random numbers from nmtst_get_rand() get seeded randomly at each start.
- *   You can specify the seed by setting NMTST_SEED_RAND. Also, tests will print the seed
- *   to stdout, so that you know the chosen seed.
+ *   Tests that use random numbers from nmtst_get_rand() get seeded at each start.
+ *   You can specify the seed by setting NMTST_SEED_RAND to a particular number or empty ("")
+ *   for a random one. If NMTST_SEED_RAND is not set (default) a stable seed gets chosen.
+ *   Tests will print the seed to stdout, so that you know which one was chosen or generated.
  *
  *
  * NMTST_DEBUG environment variable:
@@ -145,10 +146,16 @@
 		const gint64 nmtst_wait_start_us = g_get_monotonic_time (); \
 		const gint64 nmtst_wait_duration_us = (max_wait_ms) * 1000L; \
 		const gint64 nmtst_wait_end_us = nmtst_wait_start_us + nmtst_wait_duration_us; \
+		gint64 _nmtst_wait_remaining_us = nmtst_wait_duration_us; \
+		int _nmtst_wait_iteration = 0; \
 		\
 		while (TRUE) { \
+			_nm_unused const gint64 nmtst_wait_remaining_us = _nmtst_wait_remaining_us; \
+			_nm_unused int nmtst_wait_iteration = _nmtst_wait_iteration++; \
+			\
 			{ wait }; \
-			if (g_get_monotonic_time () > nmtst_wait_end_us) { \
+			_nmtst_wait_remaining_us = (nmtst_wait_end_us - g_get_monotonic_time ()); \
+			if (_nmtst_wait_remaining_us <= 0) { \
 				_not_expired = FALSE; \
 				break; \
 			} \
@@ -784,9 +791,20 @@ nmtst_get_rand (void)
 
 	if (G_UNLIKELY (!__nmtst_internal.rand)) {
 		guint32 seed;
-		const char *str;
+		const char *str = g_getenv ("NMTST_SEED_RAND");
 
-		if ((str = g_getenv ("NMTST_SEED_RAND"))) {
+		if (!str) {
+			/* No NMTST_SEED_RAND. Pick a stable one. */
+			seed = 0;
+			__nmtst_internal.rand = g_rand_new_with_seed (seed);
+		} else if (str[0] == '\0') {
+			/* NMTST_SEED_RAND is set but empty. Pick a random one. */
+			__nmtst_internal.rand = g_rand_new ();
+
+			seed = g_rand_int (__nmtst_internal.rand);
+			g_rand_set_seed (__nmtst_internal.rand, seed);
+		} else {
+			/* NMTST_SEED_RAND is set. Use it as a seed. */
 			gchar *s;
 			gint64 i;
 
@@ -795,11 +813,6 @@ nmtst_get_rand (void)
 
 			seed = i;
 			__nmtst_internal.rand = g_rand_new_with_seed (seed);
-		} else {
-			__nmtst_internal.rand = g_rand_new ();
-
-			seed = g_rand_int (__nmtst_internal.rand);
-			g_rand_set_seed (__nmtst_internal.rand, seed);
 		}
 		__nmtst_internal.rand_seed = seed;
 
